<?php

class ControllerSmartbillLogin extends Controller {
	private $error = array(); // This is used to set the errors, if any.
	 
	public function index() {   // Default function 
	    $this->load->model('setting/setting');
        $this->load->model('smartbill/common');

        $this->model_smartbill_common->validateSettingsValues();

        $this->_labels();
        $this->_breadcrumbs();
	    $this->document->setTitle($this->language->get('heading_title')); // Set the title of the page to the heading title in the Language file i.e., Smart Bill

        if ( !empty($this->request->post['submitSmartBill']) )
        {
        	try {
        		$this->saveUser();
        	} catch (Exception $ex) {
        		$this->data['warning'] = $ex->getMessage();
                // $this->data['warning'] = 'Autentificare esuata. Va rugam verificati datele si incercati din nou.';
        	}
        } else {
            if ( !$this->model_smartbill_common->isConnected() ) {
                $this->data['warning'] = 'Nu s-a reusit conectarea la Smart Bill Cloud. Verificati numele de utilizator si parola';
            }            
        }

        if ( $this->model_smartbill_common->isConnected() ) {
            $this->data['success'] = 'Conectat cu succes la Smart Bill Cloud';
        }

        $this->data += $this->model_smartbill_common->getSettings();

	    $this->load->model('design/layout'); // Loading the Design Layout Models	 
	    $this->data['layouts'] = $this->model_design_layout->getLayouts(); // Getting all the Layouts available on system	 
	    $this->template = 'smartbill/login.tpl'; // Loading the smartbill.tpl template
	    $this->children = array(
	        'common/header',
	        'common/footer'
	    );  // Adding children to our default template i.e., smartbill.tpl 
	 
	    $this->response->setOutput($this->render()); // Rendering the Output		
	}

	private function _labels() {
	    $this->language->load('module/smartbill'); // Loading the language file of smartbill 
	 
		$this->data['warning'] = '';
		$this->data['success'] = '';

	    $this->data['heading_title'] = $this->language->get('heading_title');
	 
	    $this->data['button_login'] = $this->language->get('button_login');
	    $this->data['button_save'] = $this->language->get('button_save');
	    $this->data['button_cancel'] = $this->language->get('button_cancel');
	    $this->data['button_add_module'] = $this->language->get('button_add_module');
	    $this->data['button_remove'] = $this->language->get('button_remove');		

	    $this->data['action'] = $this->url->link('smartbill/login', 'token=' . $this->session->data['token'], 'SSL'); // URL to be directed when the save button is pressed	 
	    $this->data['cancel'] = $this->url->link('smartbill/login', 'token=' . $this->session->data['token'], 'SSL'); // URL to be redirected when cancel button is pressed
	}

	private function _breadcrumbs() {
	    $this->data['breadcrumbs'] = array(
	    	array(
		        'text'      => $this->language->get('text_home'),
		        'href'      => $this->url->link('common/home', 'token=' . $this->session->data['token'], 'SSL'),
		        'separator' => false,
		    ),
		    array(
		        'text'      => $this->language->get('text_module'),
		        'href'      => $this->url->link('module/smartbill', 'token=' . $this->session->data['token'], 'SSL'),
		        'separator' => ' :: ',
		    ),
		    array(
		        'text'      => $this->language->get('heading_title'),
		        'href'      => $this->url->link('smartbill/login', 'token=' . $this->session->data['token'], 'SSL'),
		        'separator' => ' :: ',
		    )
	    );
	}

    private function saveUser() {
        $settings = $this->model_smartbill_common->getSettings();
    	$settings['SMARTBILL_USER'] = $this->request->post['smartbill_user'];
    	$this->model_setting_setting->editSetting('smartbill', $settings);

        if ( !empty($this->request->post['smartbill_pass']) ) {
        	try {
        		$this->model_smartbill_common->connectToCloud();
        	} catch (Exception $ex) {
        		throw $ex;
        	}
        }
    }
}